from jmetal import algorithm
from jmetal import core
from jmetal import operator
from jmetal import problem

__all__ = ['core', 'algorithm', 'operator', 'problem']
__version__ = '1.0'
__author__ = 'Antonio J. Nebro <ajnebro@uma.es>'
